"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("./utils");
const Builder_1 = require("./Builder");
const emd = require("ecmarkdown");
/*@internal*/
class Algorithm extends Builder_1.default {
    static enter(context) {
        context.inAlg = true;
        const { node } = context;
        // prettier-ignore
        const rawHtml = 'ecmarkdownOut' in node
            ? node.ecmarkdownOut
            : emd.algorithm(node.innerHTML);
        // replace spaces after !/? with &nbsp; to prevent bad line breaking
        const html = rawHtml.replace(/((?:\s+|>)[!?])\s+(\w+\s*\()/g, '$1&nbsp;$2');
        node.innerHTML = html;
        let labeledStepEntries = [];
        let replaces = node.getAttribute('replaces-step');
        if (replaces) {
            context.spec.replacementAlgorithms.push({
                element: node,
                target: replaces,
            });
            context.spec.replacementAlgorithmToContainedLabeledStepEntries.set(node, labeledStepEntries);
        }
        let labeledSteps = Array.from(node.querySelectorAll('li[id]'));
        if (replaces && labeledSteps.length > 0 && node.firstElementChild.children.length > 1) {
            utils_1.logWarning('You should not label a step in a replacement algorithm which has multiple top-level steps because the resulting step number could be ambiguous.');
        }
        for (const step of labeledSteps) {
            let entry = {
                type: 'step',
                id: step.id,
                stepNumbers: getStepNumbers(step),
                referencingIds: [],
            };
            context.spec.biblio.add(entry);
            if (replaces) {
                // The biblio entries for labeled steps in replacement algorithms will be modified in-place by a subsequent pass
                labeledStepEntries.push(entry);
                context.spec.labeledStepsToBeRectified.add(step.id);
            }
        }
    }
    static exit(context) {
        context.inAlg = false;
    }
}
exports.default = Algorithm;
Algorithm.elements = ['EMU-ALG'];
function getStepNumbers(item) {
    var _a;
    let counts = [];
    while (((_a = item.parentElement) === null || _a === void 0 ? void 0 : _a.tagName) === 'OL') {
        counts.unshift(1 + Array.from(item.parentElement.children).indexOf(item));
        item = item.parentElement.parentElement;
    }
    return counts;
}
