"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Builder_1 = require("./Builder");
const grammarkdown_1 = require("grammarkdown");
const endTagRe = /<\/?(emu-\w+|h?\d|p|ul|table|pre|code)\b[^>]*>/i;
const globalEndTagRe = /<\/?(emu-\w+|h?\d|p|ul|table|pre|code)\b[^>]*>/gi;
/*@internal*/
class Grammar extends Builder_1.default {
    static enter({ spec, node }) {
        if ('grammarkdownOut' in node) {
            // i.e., we already parsed this during the lint phase
            // @ts-ignore
            node.innerHTML = node.grammarkdownOut;
            return;
        }
        let content;
        let possiblyMalformed = true;
        if (spec.sourceText) {
            // If the source text is available, we should use it since `innerHTML` serializes the
            // DOM tree beneath the node. This can result in odd behavior when the syntax is malformed
            // in a way that parse5 does not understand, but grammarkdown could possibly recover.
            const location = spec.dom.nodeLocation(node);
            if (location) {
                if (location.startTag && location.endTag) {
                    // the parser was able to find a matching end tag.
                    const start = location.startTag.endOffset;
                    const end = location.endTag.startOffset;
                    content = spec.sourceText.slice(start, end);
                }
                else {
                    // TODO this is not reached
                    // the parser was *not* able to find a matching end tag. Try to recover by finding a
                    // possible end tag, otherwise read the rest of the source text.
                    const start = (globalEndTagRe.lastIndex = location.endOffset);
                    const match = globalEndTagRe.exec(spec.sourceText);
                    const end = match ? match.index : spec.sourceText.length;
                    content = spec.sourceText.slice(start, end);
                    // since we already tested for an end tag, no need to test again later.
                    possiblyMalformed = false;
                    globalEndTagRe.lastIndex = 0;
                }
            }
            else {
                // TODO this is not reached
                // can't read location for whatever reason, so fallback to innerHTML
                content = node.innerHTML;
            }
        }
        else {
            // no source text, so read innerHTML as a fallback.
            content = node.innerHTML;
        }
        if (possiblyMalformed) {
            // check for a possible end-tag in the content. For now we only check for a few possible
            // recovery cases, namely emu-* tags, and a few block-level elements.
            const match = endTagRe.exec(content);
            if (match) {
                content = content.slice(0, match.index);
            }
        }
        const options = {
            format: grammarkdown_1.EmitFormat.ecmarkup,
            noChecks: true,
        };
        node.innerHTML = grammarkdown_1.Grammar.convert(content, options, 
        /*hostFallback*/ undefined, spec.cancellationToken);
    }
}
exports.default = Grammar;
Grammar.elements = ['EMU-GRAMMAR'];
