"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const args_1 = require("./args");
const args = args_1.argParser.parse();
const fs = require("fs");
const ecmarkup = require("./ecmarkup");
const utils = require("./utils");
const debounce = require('promise-debounce');
// back compat to old argument names
if (args.css) {
    args.cssOut = args.css;
}
if (args.js) {
    args.jsOut = args.js;
}
if (args.lintSpec && args.watch) {
    console.error('Cannot use --lint-spec with --watch');
    process.exit(1);
}
const watching = new Map();
const build = debounce(function build() {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const opts = Object.assign({}, args);
            if (args.lintSpec) {
                let descriptor = `eslint/lib/cli-engine/formatters/${args.lintFormatter}.js`;
                try {
                    require.resolve(descriptor);
                }
                catch (_a) {
                    descriptor = args.lintFormatter;
                }
                let formatter = require(descriptor);
                opts.reportLintErrors = (errors, sourceText) => {
                    if (errors.length < 1)
                        return;
                    let results = [
                        {
                            filePath: args.infile,
                            // for now, everything is an error (2 in eslint terms)
                            messages: errors.map(e => (Object.assign({ severity: 2 }, e))),
                            errorCount: errors.length,
                            warningCount: 0,
                            // for now, nothing is fixable
                            fixableErrorCount: 0,
                            fixableWarningCount: 0,
                            source: sourceText,
                        },
                    ];
                    console.error(formatter(results));
                    process.exit(1);
                };
            }
            const spec = yield ecmarkup.build(args.infile, utils.readFile, opts);
            const pending = [];
            if (args.biblio) {
                if (args.verbose) {
                    utils.logVerbose('Writing biblio file to ' + args.biblio);
                }
                pending.push(utils.writeFile(args.biblio, JSON.stringify(spec.exportBiblio())));
            }
            if (args.outfile) {
                if (args.verbose) {
                    utils.logVerbose('Writing output to ' + args.outfile);
                }
                pending.push(utils.writeFile(args.outfile, spec.toHTML()));
            }
            else {
                process.stdout.write(spec.toHTML());
            }
            yield Promise.all(pending);
            if (args.watch) {
                const toWatch = new Set(spec.imports.map(i => i.importLocation).concat(args.infile));
                // remove any files that we're no longer watching
                for (const [file, watcher] of watching) {
                    if (!toWatch.has(file)) {
                        watcher.close();
                        watching.delete(file);
                    }
                }
                // watch any new files
                for (const file of toWatch) {
                    if (!watching.has(file)) {
                        watching.set(file, fs.watch(file, build));
                    }
                }
            }
        }
        catch (e) {
            if (args.watch) {
                process.stderr.write(e.stack);
            }
            else {
                throw e;
            }
        }
    });
});
build().catch(e => {
    console.error(e);
    process.exit(1);
});
