"use strict";
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
const ecmarkdown_1 = require("ecmarkdown");
const utils_1 = require("./utils");
const algorithm_line_endings_1 = require("./rules/algorithm-line-endings");
const algorithm_step_numbering_1 = require("./rules/algorithm-step-numbering");
const algorithm_step_labels_1 = require("./rules/algorithm-step-labels");
let algorithmRules = [
    algorithm_line_endings_1.default,
    algorithm_step_numbering_1.default,
    algorithm_step_labels_1.default,
];
function composeObservers(...observers) {
    return {
        enter(node) {
            var _a;
            for (let observer of observers) {
                (_a = observer.enter) === null || _a === void 0 ? void 0 : _a.call(observer, node);
            }
        },
        exit(node) {
            var _a;
            for (let observer of observers) {
                (_a = observer.exit) === null || _a === void 0 ? void 0 : _a.call(observer, node);
            }
        },
    };
}
function collectAlgorithmDiagnostics(dom, sourceText, algorithms) {
    let lintingErrors = [];
    for (let algorithm of algorithms) {
        let element = algorithm.element;
        let location = utils_1.getLocation(dom, element);
        if (location.endTag == null) {
            lintingErrors.push({
                ruleId: 'missing-close-tag',
                message: 'could not find closing tag for emu-alg',
                line: location.startTag.line,
                column: location.startTag.col,
                nodeType: 'EMU-ALG',
            });
            continue;
        }
        let reporter = (_a) => {
            var { line, column } = _a, others = __rest(_a, ["line", "column"]);
            // jsdom's lines and columns are both 1-based
            // ecmarkdown has 1-based line numbers and 0-based column numbers
            // we want 1-based for both
            let trueLine = location.startTag.line + line - 1;
            let trueCol = column;
            if (line === 1) {
                trueCol +=
                    location.startTag.col + (location.startTag.endOffset - location.startTag.startOffset);
            }
            else {
                trueCol += 1;
            }
            lintingErrors.push(Object.assign({ line: trueLine, column: trueCol }, others));
        };
        let algorithmSource = sourceText.slice(location.startTag.endOffset, location.endTag.startOffset);
        let observer = composeObservers(...algorithmRules.map(f => f(reporter, element, algorithmSource)));
        let tree = ecmarkdown_1.parseAlgorithm(algorithmSource, { trackPositions: true });
        ecmarkdown_1.visit(tree, observer);
        algorithm.tree = tree;
    }
    return lintingErrors;
}
exports.collectAlgorithmDiagnostics = collectAlgorithmDiagnostics;
