"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ruleId = 'algorithm-numbering';
/*
Checks that list numbers are all `1.`, with the exception of top-level lists whose first item is not that.
*/
function default_1(report, node, algorithmSource) {
    const nodeType = node.tagName;
    let depth = -1;
    let topLevelIsNotOne = false;
    return {
        enter(node) {
            if (node.name === 'ol') {
                ++depth;
                if (depth === 0) {
                    topLevelIsNotOne = node.start !== 1;
                }
            }
            else if (node.name === 'ordered-list-item') {
                if (depth === 0 && topLevelIsNotOne) {
                    return;
                }
                let itemSource = algorithmSource.slice(node.location.start.offset, node.location.end.offset);
                let match = itemSource.match(/^([ \t]*)([^.]+)\. /);
                if (match[2] !== '1') {
                    report({
                        ruleId,
                        nodeType,
                        line: node.location.start.line,
                        column: node.location.start.column + match[1].length,
                        message: 'expected line number to be "1."',
                    });
                }
            }
        },
        exit(node) {
            if (node.name === 'ol') {
                --depth;
            }
        }
    };
}
exports.default = default_1;
//# sourceMappingURL=algorithm-numbering.js.map